@@ -1,6 +1,7 @@
 import torch.nn as nn
 import torch
 
+from .image_transforms import resize_4d_tensor_by_size
 
 class CaricatureLoss(nn.Module):
 
@@ -20,7 +21,16 @@ class CaricatureLoss(nn.Module):
         or find me on github: 
             github.com/mayukhdeb
         """
-        numerator = (x*y).sum() 
+
+        if x.shape != y.shape:
+            """
+            if their shapes are not equal (likely due to using static caricatures), then resize the target accordingly
+            """
+            y = resize_4d_tensor_by_size(y.unsqueeze(0), height = x.shape[-2], width = x.shape[-1] ).squeeze(0)
+
+        y = y.detach()
+        
+        numerator = (x*y.detach()).sum() 
         denominator = torch.sqrt((y**2).sum()) + eps
         cossim = numerator/denominator
         cossim = torch.maximum(torch.tensor(0.1).to(cossim.device), cossim)
